{BaseStage} = require('./base')
functions = require('./../functions')
udfs = require('./../udfs')
nodes = require('sql-parser').nodes


exports.Project = class Project extends BaseStage

  constructor: (fields) ->
    @fields = fields
    @initFunctions()
  
  push: (data) ->
    projectedData = @extractFieldsFromRecord(data)
    @nextStage.push(projectedData)
    
  isStarQuery: -> 
    @fields.length is 1 and @fields[0].star
    
  extractFieldsFromRecord: (record) ->
    return record if @isStarQuery()
    ret = {}
    for field in @fields
      ret[@fieldName(field)] = @fieldValue(field, record)
    ret
  
  fieldName: (field) ->
    if @fieldIsFunction(field.field)
      field.name or field.toString()
    else
      field.name or field.field.value
  
  fieldValue: (field, record) ->
    if @fieldIsFunction(field.field)
      fn = @functions[@fieldName(field)]
      if field.field.udf
        
        fn.apply(record, @buildFnArgs(field.field.arguments, record))
      else
        fn.push(record)
    else
      record[field.field.value]
  
  fieldIsFunction: (field) -> 
    field? and field.constructor is nodes.FunctionValue
  
  initFunctions: () ->
    @functions = {}
    for field in @fields when @fieldIsFunction(field.field)
      if field.field.udf
        @functions[@fieldName(field)] = udfs.get(field.field.name)
      else
        klass = functions.get(field.field.name)
        instance = new klass(field.field.arguments)
        @functions[@fieldName(field)] = instance
      
      
  buildFnArgs: (args, record) ->
    fnArgs = []
    for arg in args
      switch arg.constructor 
        when nodes.NumberValue  then arg.value
        when nodes.LiteralValue then record[arg.value]
        else arg.value